import os
from modi_helper.utils.job import run


def create_environment(
    name,
    destination=None,
    automatic_yes=False,
    quiet=False,
    extra_conda_args=None,
):
    command = ["conda", "create"]
    if extra_conda_args:
        if isinstance(extra_conda_args, (list, tuple, set)):
            command.extend(extra_conda_args)
        elif isinstance(extra_conda_args, str):
            extra_conda_args_list = extra_conda_args.split(" ")
            command.extend(extra_conda_args_list)

    if destination:
        command.extend(["-p", os.path.join(destination, name)])
    else:
        command.extend(["-n", name])

    if automatic_yes:
        command.extend(["-y"])

    if quiet:
        command.extend(["-q"])

    return run(command, format_output_str=True, capture_output=quiet)


def activate_environment(name):
    command = ["conda", "activate", name]
    return run(command, format_output_str=True)


def add_environment_directory(path):
    command = ["conda", "config", "--append", "envs_dirs", path]
    return run(command, format_output_str=True)



import click
import os
import sys
from modi_helper.utils.io import exists, makedirs
from modi_helper.environment.initialize import initialize_conda
from modi_helper.environment.create import (
    create_environment,
    activate_environment,
    add_environment_directory,
)
from modi_helper.cli.return_codes import (
    SUCCESS,
    WRITE_ERROR,
    SETUP_ERROR,
    EXECUTE_ERROR,
)


@click.command(context_settings=dict(help_option_names=["-h", "--help"]))
@click.argument("name", type=click.STRING)
@click.option(
    "--destination-dir",
    "-dd",
    default=os.path.join("~", "modi_mount", "my_conda_environments"),
    show_default=True,
    help="The directory in which the environment will be created",
)
@click.option(
    "--automatic-yes",
    "-y",
    default=True,
    is_flag=True,
    help="Whether the environment creation should automatically proceed without user input.",
)
@click.option(
    "--activate", "-a", is_flag=True, default=True, show_default=True
)
@click.option("--quiet", "-q", is_flag=True, default=False, show_default=True)
# Make a click option for extra conda args
@click.option(
    "--extra-conda-args",
    default=None,
    show_default=True,
    help="Extra arguments to pass to conda",
)
def main(
    name,
    destination_dir,
    automatic_yes,
    activate,
    quiet,
    extra_conda_args,
):
    if not exists(destination_dir):
        created, msg = makedirs(destination_dir)
        if not created:
            print(msg)
            return WRITE_ERROR

    # Ensure that conda is initialized
    initialized, output = initialize_conda(quiet=quiet)
    if not initialized:
        print(output)
        return SETUP_ERROR

    created = create_environment(
        name,
        destination=destination_dir,
        automatic_yes=automatic_yes,
        quiet=quiet,
        extra_conda_args=extra_conda_args,
    )
    if created["returncode"] != "0":
        print("Failed to create environment: {} - {}".format(name, created))
        return SETUP_ERROR

    if activate:
        activated = activate_environment(name)
        if not activated:
            print(
                "Failed to activate environment: {} - {}".format(
                    name, activated
                )
            )
            return EXECUTE_ERROR

    # Ensure that the destionation directory is added as a conda environment directory
    # This is done to ensure that the environment can be activated from anywhere
    # in the file system by name.
    added = add_environment_directory(destination_dir)
    if not added:
        print(
            "Failed to add environment directory: {} - {}".format(name, added)
        )
        return SETUP_ERROR

    return SUCCESS


def cli():
    sys.exit(main())


if __name__ == "__main__":
    sys.exit(main())



import os
import json
from modi_helper.utils.job import run


def list_environments(extra_conda_args=None):
    command = ["conda", "env", "list", "--json"]
    if extra_conda_args:
        if isinstance(extra_conda_args, (list, tuple, set)):
            command.extend(extra_conda_args)
        elif isinstance(extra_conda_args, str):
            extra_conda_args_list = extra_conda_args.split(" ")
            command.extend(extra_conda_args_list)

    environment_results = run(
        command, format_output_str=True, capture_output=True
    )
    if not environment_results:
        return False, []

    if "error" in environment_results and environment_results["error"]:
        print(
            "Failed to list the environments, error: {}".format(
                environment_results["error"]
            )
        )
        return False, []

    if (
        "returncode" in environment_results
        and environment_results["returncode"] != "0"
    ):
        print(
            "Failed to list the environments, returncode: {}".format(
                environment_results["returncode"]
            )
        )
        return False, []

    if (
        "output" not in environment_results
        or not environment_results["output"]
    ):
        print(
            "Failed to list the environments, output: {}".format(
                environment_results
            )
        )
        return False, []

    json_output = json.loads(environment_results["output"])
    environments = []
    for environment in json_output["envs"]:
        new_environment = {}
        environment_name = os.path.basename(environment)
        new_environment["name"] = environment_name
        new_environment["path"] = environment
        environments.append(new_environment)

    return True, environments



import click
import sys
from modi_helper.environment.initialize import initialize_conda
from modi_helper.environment.list import list_environments
from modi_helper.cli.return_codes import (
    SUCCESS,
    EXECUTE_ERROR,
    SETUP_ERROR,
)


@click.command(context_settings=dict(help_option_names=["-h", "--help"]))
# Make a click option for extra conda args
@click.option(
    "--extra-conda-args",
    default=None,
    show_default=True,
    help="Extra arguments to pass to conda",
)
def main(extra_conda_args):
    # Ensure that conda is initialized
    initialized, output = initialize_conda(quiet=True)
    if not initialized:
        print(output)
        return SETUP_ERROR

    success, environments = list_environments(
        extra_conda_args=extra_conda_args
    )
    if not success:
        print("Failed to list environments")
        return EXECUTE_ERROR
    print(environments)
    return SUCCESS


def cli():
    sys.exit(main())


if __name__ == "__main__":
    sys.exit(main())



from modi_helper.utils.job import run


def delete_environment(name, automatic_yes=False, quiet=False):
    command = ["conda", "remove", "--name", name, "--all"]

    if automatic_yes:
        command.extend(["-y"])

    if quiet:
        command.extend(["-q"])

    return run(command, format_output_str=True, capture_output=quiet)



import click
import os
import sys
from modi_helper.defaults import CONTAINER_WRAP, REGULAR
from modi_helper.job.initialize import (
    check_job_paths,
    write_job_script,
    make_job_script_content,
    extract_extra_job_settings,
)
from modi_helper.job.run import run_job
from modi_helper.utils.io import (
    exists,
    expanduser,
    set_execute_permissions,
)
from modi_helper.cli.return_codes import (
    SUCCESS,
    PATH_NOT_FOUND,
    PERMISSIONS_ERROR,
    SETUP_ERROR,
    WRITE_ERROR,
    EXECUTE_ERROR,
)


@click.command(
    "cli",
    context_settings=dict(
        help_option_names=["-h", "--help"], show_default=True
    ),
)
@click.argument("job-file")
@click.option(
    "--job-runner",
    "-jr",
    default="srun",
    help="""
    The executable that is used to execute the container image if the --generate-container-wrap is used.
    """,
)
@click.option(
    "--job-args",
    "-ja",
    multiple=True,
    default=[],
    help="""
    The list of arguments that should be passed to the JOB_FILE.
    If multiple arguments are to be passed, then the `--job-args` flag should be used multiple times, where each of which should contain a single argument.
    For instance, to pass the arguments: "arg1 arg2 arg3", the following syntax should be used: --job-args arg1 --job-args arg2 --job-args arg3.
    """,
)
@click.option(
    "--runtime-directory",
    "-rd",
    default=os.getcwd(),
    help="""The path to the runtime directory in which the job is to be executed.
    This directory must be within the scratch space directory.""",
)
@click.option(
    "--scratch-space-directory",
    "-ssd",
    default=os.path.join("~", "modi_mount"),
    help="""The path to the scratch space directory.""",
)
@click.option(
    "--generate-job-scripts",
    "-gjs",
    default=False,
    is_flag=True,
    help="""
    Whether the script should generate job script files that execute the JOB_FILE as a job.
    """,
)
@click.option(
    "--generate-container-wrap",
    "-gcw",
    default=False,
    is_flag=True,
    help="""
    Whether the generate job scripts should wrap the JOB_FILE execution in a container environment.
    """,
)
@click.option(
    "--container-wrap-image",
    "-cwi",
    default=os.path.join(
        "~",
        "modi_images",
        os.getenv("MODI_DEFAULT_IMAGE", "ucphhpc/slurm-notebook:latest"),
    ),
    help="""
    The container image to use when generating the job scripts.
    """,
)
@click.option(
    "--verbose",
    "-v",
    default=False,
    is_flag=True,
    help="""
    Whether to print verbose output about how the JOB_FILE is submitted and with what settings.
    """,
)
def main(
    job_file,
    job_runner,
    job_args,
    runtime_directory,
    scratch_space_directory,
    generate_job_scripts,
    generate_container_wrap,
    container_wrap_image,
    verbose,
):
    job_file = expanduser(job_file)
    runtime_directory = expanduser(runtime_directory)
    scratch_space_directory = expanduser(scratch_space_directory)

    # Prepend the current working directory to the JOB_FILE path if no subpath is specified
    if not os.path.dirname(job_file):
        job_file = os.path.join(os.getcwd(), job_file)

    if not exists(job_file):
        print(
            "Failed to find the JOB_FILE:'{}' are you sure it exists?".format(
                job_file
            )
        )
        return PATH_NOT_FOUND

    if not os.access(job_file, os.X_OK):
        print(
            "The JOB_FILE:'{}' does not have the executable permission set.".format(
                job_file
            )
        )
        # Set execute permissions on the JOB_FILE
        print(
            "Trying to set execute permissions on the JOB_FILE: {}".format(
                job_file
            )
        )
        if not set_execute_permissions(job_file):
            return PERMISSIONS_ERROR
        print(
            "Succeeded in giving the JOB_FILE: {} execute permissions.".format(
                job_file
            )
        )

    if not exists(runtime_directory):
        print(
            "The specified runtime directory: '{}' does not exist.".format(
                runtime_directory
            )
        )
        return PATH_NOT_FOUND

    if not exists(scratch_space_directory):
        print(
            "The specified scratch space directory: '{}' does not exist.".format(
                scratch_space_directory
            )
        )
        return PATH_NOT_FOUND

    correct_directories = check_job_paths(
        scratch_space_directory, runtime_directory
    )
    if not correct_directories:
        print(
            "Your `runtime-directory`: {} must reside inside the `scratch-space-directory`: {}".format(
                runtime_directory, scratch_space_directory
            )
        )
        return SETUP_ERROR

    correct_job = check_job_paths(runtime_directory, job_file)
    if not correct_job:
        print(
            "Your JOB_FILE: {} must reside inside the `runtime-directory`: {}".format(
                job_file, runtime_directory
            )
        )
        return SETUP_ERROR

    if verbose:
        print(
            "Submitting JOB_FILE: {} with the following settings".format(
                job_file
            )
        )
        print("Job Runner: {}".format(job_runner))
        print("Job Args: {}".format(job_args))
        print("Runtime Directory: {}".format(runtime_directory))
        print("Scratch Space Directory: {}".format(scratch_space_directory))
        if generate_job_scripts:
            print("Generate Job Scripts: {}".format(generate_job_scripts))
        if generate_container_wrap:
            print(
                "Generate Container Wrap: {}".format(generate_container_wrap)
            )
            print("Container Wrap Image: {}".format(container_wrap_image))

    if generate_job_scripts:
        # We need to expand the job_args to a string so that we can pass it to the template_kwargs
        template_kwargs = {
            "job_runner": job_runner,
            "job_file": job_file,
            "job_args": " ".join(job_args),
        }
        if generate_container_wrap:
            template_file_name = CONTAINER_WRAP + ".j2"
            new_job_file_name = "{}.{}".format(
                os.path.basename(job_file), CONTAINER_WRAP
            )
            template_kwargs["container_wrap_image"] = container_wrap_image
        else:
            template_file_name = REGULAR + ".j2"
            new_job_file_name = "{}.{}".format(
                os.path.basename(job_file), REGULAR
            )

        # Check if the original JOB_FILE sets extra_job_settings
        # If so, then we need to pass these to the new job script
        # so that the new job script can set the same settings.
        extra_job_settings = extract_extra_job_settings(job_file)
        if extra_job_settings:
            template_kwargs["extra_job_settings"] = extra_job_settings

        new_job_file_path = os.path.join(runtime_directory, new_job_file_name)
        job_script_content = make_job_script_content(
            template_file_name, template_kwargs=template_kwargs
        )
        wrote_job_script = write_job_script(
            new_job_file_path, job_script_content
        )
        if not wrote_job_script:
            print(
                "Failed to write the generated job script: {}".format(
                    new_job_file_name
                )
            )
            return WRITE_ERROR
        if verbose:
            print(
                "Successfully generated new job script: {} from template: {}".format(
                    new_job_file_name, template_file_name
                )
            )
        # Ensure the new job script has the executable permission set
        if not set_execute_permissions(new_job_file_path):
            print(
                "Failed to set the execute permission on generated new job script: {}".format(
                    new_job_file_path
                )
            )
            return PERMISSIONS_ERROR
        job_file = new_job_file_path

    if verbose:
        print(
            "Executing JOB_FILE: {} with the following arguments: {}".format(
                job_file, job_args
            )
        )
    if generate_job_scripts:
        # When generating job scripts, the job arguments have been passed to the template_kwargs
        # which has generated a new job script with the job arguments.
        # Therefore, we do not need to pass the job_args to the run_job function since it is already included in the new job script.
        job_output = run_job(runtime_directory, job_file)
    else:
        job_output = run_job(runtime_directory, job_file, *job_args)
    print(
        "Your job output will be placed in the runtime directory: {}".format(
            runtime_directory
        )
    )
    if job_output["returncode"] != "0":
        print(
            "Failed to execute the JOB_FILE: {} - {}".format(
                job_file, job_output
            )
        )
        return EXECUTE_ERROR
    if verbose:
        print("Job executed successfully.")
    return SUCCESS


def cli():
    sys.exit(main())


if __name__ == "__main__":
    sys.exit(main())



# Generate a unit test file for the submit_job.py script.
import os
import unittest
from modi_helper.utils.job import run

TEST_RESOURCE_DIR = os.path.join(os.path.dirname(__file__), "res")


class TestNewEnvironment(unittest.TestCase):
    def setUp(self) -> None:
        self.new_environment_name = "test_environment"
        return super().setUp()

    def tearDown(self) -> None:
        return super().tearDown()

    def test_new_environment(self):
        args = [
            "modi-new-environment",
            self.new_environment_name,
            "-dd",
            TEST_RESOURCE_DIR,
            "--automatic-yes",
        ]
        result = run(args, format_output_str=True)
        assert result["output"] == ""


class TestLoadEnvironment(unittest.TestCase):
    def setUp(self) -> None:
        return super().setUp()

    def tearDown(self) -> None:
        return super().tearDown()

    def test_load_environment(self):
        args = [
            "modi-load-environments",
            "-q",
            TEST_RESOURCE_DIR,
        ]
        result = run(args, format_output_str=True)
        assert result["output"] == ""



import click
import os
import sys
from modi_helper.utils.io import exists
from modi_helper.environment.initialize import (
    get_environment_directories,
)
from modi_helper.environment.create import (
    add_environment_directory,
)
from modi_helper.cli.return_codes import (
    PATH_NOT_FOUND,
    SUCCESS,
    SETUP_ERROR,
)


@click.command(context_settings=dict(help_option_names=["-h", "--help"]))
@click.option(
    "--environment-dir",
    "-ed",
    default=os.path.join("~", "modi_mount", "my_conda_environments"),
    show_default=True,
    help="The directory in which the environments are located.",
)
@click.option("--quiet", "-q", is_flag=True, default=False, show_default=True)
def main(environment_dir, quiet):
    if not exists(environment_dir):
        print("The designated environment directory does not exist")
        return PATH_NOT_FOUND

    found, environment_directories = get_environment_directories()
    if found:
        if environment_dir in environment_directories:
            if not quiet:
                print(
                    "The environment directory: {} is already in the list of environment directories: {}".format(
                        environment_dir, environment_directories
                    )
                )
            return SUCCESS

    # Ensure that the destionation directory is added as a conda environment directory
    # This is done to ensure that the environment can be activated from anywhere
    # in the file system by name.
    added = add_environment_directory(environment_dir)
    if not added:
        print(
            "Failed to add environment directory: {} - {}".format(
                environment_dir, added
            )
        )
        return SETUP_ERROR
    return SUCCESS


def cli():
    sys.exit(main())


if __name__ == "__main__":
    sys.exit(main())



import os
import shutil


def makedirs(path):
    try:
        os.makedirs(expanduser(path))
        return True, "Created: {}".format(path)
    except IOError as err:
        return (
            False,
            "Failed to create the directory path: {} - {}".format(path, err),
        )


def write(path, content, mode="w", mkdirs=False):
    dir_path = os.path.dirname(path)
    if not exists(dir_path) and mkdirs:
        if not makedirs(dir_path):
            return False
    try:
        with open(path, mode) as fh:
            fh.write(content)
        return True
    except IOError as err:
        print("Failed to save file: {} - {}".format(path, err))
    return False


def copy(file_path, destination_directory):
    try:
        shutil.copy(file_path, destination_directory)
        return True
    except IOError as err:
        print("Failed to copy file: {} - {}".format(file_path, err))
    return False


def copy_recursive(source_directory, destination_directory):
    try:
        shutil.copytree(source_directory, destination_directory)
        return True
    except IOError as err:
        print(
            "Failed to copy directory: {} - {}".format(source_directory, err)
        )
    return False


def load(path, mode="r", readlines=False):
    try:
        with open(path, mode) as fh:
            if readlines:
                return fh.readlines()
            return fh.read()
    except IOError as err:
        print("Failed to load file: {} - {}".format(path, err))
    return False


def remove(path):
    try:
        if os.path.exists(path):
            os.remove(path)
            return True
    except IOError as err:
        print("Failed to remove file: {} - {}".format(path, err))
    return False


def removedirs(path):
    try:
        if os.path.exists(path):
            os.removedirs(path)
            return True
    except IOError as err:
        print("Failed to remove directory: {} - {}".format(path, err))
    return False


def recursive_remove(path):
    try:
        if os.path.exists(path):
            shutil.rmtree(path)
            return True
    except IOError as err:
        print("Failed to remove directory: {} - {}".format(path, err))
    return False


def exists(path):
    return os.path.exists(expanduser(path))


def normpath(path):
    return os.path.normpath(expanduser(path))


def expanduser(path):
    return os.path.expanduser(path)


def load_content(path):
    try:
        with open(path, "r") as fh:
            return fh.read()
    except IOError as err:
        print("Failed to read file: {} - {}".format(path, err))
    return False


def set_execute_permissions(path):
    try:
        os.chmod(path, 0o755)
        return True
    except IOError as err:
        print(
            "Failed to set execute permissions on file: {} - {}".format(
                path, err
            )
        )
    return False



import os
from jinja2 import Environment, PackageLoader, select_autoescape
import importlib.resources as pkg_resources
from modi_helper.utils.io import expanduser, write, load


def check_job_paths(runtime_directory, job_file):
    """The job_file must exist within the runtime_directory.
    If not, the cluster nodes will not be able to access it."""
    return expanduser(job_file).startswith(expanduser(runtime_directory))


def get_package_top_directory(package_name):
    package_module = __import__(package_name)
    package_file = package_module.__file__
    top_directory = os.path.dirname(package_file)
    return top_directory


# Import the template files from the src/templates directory using pkg_resources.files.
def get_template_file(template_type):
    """Get the template file from the src/templates directory."""
    return pkg_resources.files("modi_helper.templates").joinpath(
        "{}.j2".format(template_type)
    )


def extract_extra_job_settings(job_file):
    """Extract the extra job settings from the given job file."""
    if not job_file:
        return False

    job_file_lines = load(job_file, readlines=True)

    extra_job_settings = []
    for line in job_file_lines:
        if line.startswith("#SBATCH"):
            extra_job_settings.append(line)
    return extra_job_settings


def make_job_script_content(template_name, template_kwargs=None):
    """Generate the job sripts based on the given template and its associated kwargs."""
    if not template_kwargs:
        return False

    env = Environment(
        loader=PackageLoader("modi_helper"),
        autoescape=select_autoescape(),
    )
    template = env.get_template(template_name)
    new_content = template.render(
        **template_kwargs
    )  # this is where to put args to the template renderer
    return new_content


def write_job_script(path, content):
    """
    Generate the set of job scripts that are required to execute
    a given binary/program.
    """
    return write(path, content, mode="w", mkdirs=True)



import click
import sys
from modi_helper.environment.initialize import initialize_conda
from modi_helper.environment.delete import delete_environment
from modi_helper.cli.return_codes import (
    SETUP_ERROR,
    EXECUTE_ERROR,
    SUCCESS,
)


@click.command(context_settings=dict(help_option_names=["-h", "--help"]))
@click.argument("name", type=click.STRING)
@click.option(
    "--automatic-yes",
    "-y",
    default=True,
    help="Whether the environment deletion should automatically proceed without user input.",
)
@click.option("--quiet", "-q", is_flag=True, default=False, show_default=True)
def main(name, automatic_yes, quiet):
    # Ensure that conda is initialized
    initialized, output = initialize_conda(quiet=quiet)
    if not initialized:
        print(output)
        return SETUP_ERROR

    deleted = delete_environment(
        name,
        automatic_yes=automatic_yes,
        quiet=quiet,
    )
    if deleted["returncode"] != "0":
        print("Failed to delete environment: {} - {}".format(name, deleted))
        return EXECUTE_ERROR
    return SUCCESS


def cli():
    sys.exit(main())


if __name__ == "__main__":
    sys.exit(main())



# Generate a unit test file for the submit_job.py script.
import os
import unittest
from modi_helper.utils.job import run
from modi_helper.utils.io import (
    write,
    remove,
    exists,
    makedirs,
    copy,
    recursive_remove,
)

TEST_RESOURCE_DIR = os.path.join(os.path.dirname(__file__), "res")


class TestSubmitJob(unittest.TestCase):
    def setUp(self):
        self.hello_world_job_file_name = "hello_world.sh"
        self.hello_world_python_name = "hello_world.py"

        self.hello_world_job_file = os.path.join(
            TEST_RESOURCE_DIR, self.hello_world_job_file_name
        )
        self.hello_world_python = os.path.join(
            TEST_RESOURCE_DIR, self.hello_world_python_name
        )

        # Create the runtime and scratch space directories
        self.test_scratch_space_directory = os.path.join(
            TEST_RESOURCE_DIR, "scratch"
        )
        self.test_runtime_directory = os.path.join(
            self.test_scratch_space_directory, "runtime"
        )
        created, msg = makedirs(self.test_scratch_space_directory)
        self.assertTrue(created, msg)
        created, msg = makedirs(self.test_runtime_directory)
        self.assertTrue(created, msg)

        # Install the cli
        args = ["pip3", "install", ".", "-q"]
        result = run(args)
        self.assertIsNotNone(result)
        self.assertIn("returncode", result)
        self.assertEqual(result["returncode"], 0)
        return super().setUp()

    def tearDown(self):
        if exists(self.hello_world_job_file):
            assert remove(self.hello_world_job_file)
        if exists(self.hello_world_python):
            assert remove(self.hello_world_python)

        if exists(self.test_runtime_directory):
            self.assertTrue(recursive_remove(self.test_runtime_directory))
        if exists(self.test_scratch_space_directory):
            self.assertTrue(
                recursive_remove(self.test_scratch_space_directory)
            )
        return super().tearDown()

    def test_submit_job(self):
        self.assertTrue(
            write(
                self.hello_world_job_file,
                "#!/bin/bash\necho 'Hello World!'",
            )
        )
        self.assertTrue(
            copy(self.hello_world_job_file, self.test_runtime_directory)
        )

        runtime_job_file_path = os.path.join(
            self.test_runtime_directory,
            self.hello_world_job_file_name,
        )
        self.assertTrue(exists(runtime_job_file_path))

        args = [
            "modi-new-job",
            runtime_job_file_path,
            "--scratch-space-directory",
            self.test_scratch_space_directory,
            "--runtime-directory",
            self.test_runtime_directory,
        ]
        result = run(args, format_output_str=True)
        assert result["output"] == "Hello World!\n"

    def test_submit_job_with_generate_job_scripts(self):
        self.assertTrue(
            write(
                self.hello_world_python,
                "#!/usr/bin/env python3\nprint('Hello World!')",
            )
        )
        self.assertTrue(
            copy(self.hello_world_python, self.test_runtime_directory)
        )

        runtime_job_file_path = os.path.join(
            self.test_runtime_directory, self.hello_world_python_name
        )
        self.assertTrue(exists(runtime_job_file_path))

        args = [
            "modi-new-job",
            runtime_job_file_path,
            "--generate-job-scripts",
            "--scratch-space-directory",
            self.test_scratch_space_directory,
            "--runtime-directory",
            self.test_runtime_directory,
        ]
        result = run(args, format_output_str=True)
        assert result["output"] == "Hello World!\n"



import os
from setuptools import setup, find_packages

cur_dir = os.path.abspath(os.path.dirname(__file__))


def read(path):
    with open(path, "r") as _file:
        return _file.read()


def read_req(name):
    path = os.path.join(cur_dir, name)
    return [req.strip() for req in read(path).splitlines() if req.strip()]


version_ns = {}
with open(os.path.join(cur_dir, "version.py")) as f:
    exec(f.read(), {}, version_ns)


long_description = open("README.rst").read()
setup(
    name="modi-helper",
    version=version_ns["__version__"],
    description="A set of scripts and tools to help configure a MODI environment.",
    long_description=long_description,
    author="Rasmus Munk",
    author_email="munk1@live.dk",
    packages=find_packages(),
    package_data={"modi_helper": ["templates/*.j2"]},
    package_dir={"modi_helper": "modi_helper"},
    url="https://github.com/ucphhpc/modi-helper-scripts",
    keywords=[],
    install_requires=read_req("requirements.txt"),
    extras_require={
        "dev": read_req("requirements-dev.txt"),
    },
    entry_points={
        "console_scripts": [
            "modi-new-job=modi_helper.cli.job_submit:cli",
            "modi-new-environment=modi_helper.cli.environment_new:cli",
            "modi-delete-environment=modi_helper.cli.environment_delete:cli",
            "modi-load-environments=modi_helper.cli.environments_load:cli",
            "modi-list-environments=modi_helper.cli.environments_list:cli",
        ]
    },
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
)



from modi_helper.utils.job import run


def run_job(runtime_directory, job_file, *args, **kwargs):
    command = ["sbatch", job_file]
    if args:
        command.extend(args)
    return run(command, cwd=runtime_directory, format_output_str=True)



import os
import json
from modi_helper.utils.job import run


def initialize_conda(quiet=False):
    conda_dir = os.getenv("CONDA_DIR", None)
    if not conda_dir:
        return (
            False,
            "The CONDA_DIR environment variable was not set, could not initialize conda",
        )

    # Source the conda script into the current shell
    command = ["conda", "init", "--all"]
    if quiet:
        command.extend(["-q"])
    return True, run(command, format_output_str=False, capture_output=quiet)


def get_environment_directories():
    command = ["conda", "config", "--get", "envs_dirs", "--json"]
    environment_dir_result = run(
        command, capture_output=True, format_output_str=True
    )
    if not environment_dir_result:
        print(
            "Failed to get the environment directories, result: {}".format(
                environment_dir_result
            )
        )
        return False, []

    if "error" in environment_dir_result and environment_dir_result["error"]:
        print(
            "Failed to get the environment directories, error: {}".format(
                environment_dir_result["error"]
            )
        )
        return False, []

    if (
        "returncode" in environment_dir_result
        and environment_dir_result["returncode"] != "0"
    ):
        print(
            "Failed to get the environment directories, returncode: {}".format(
                environment_dir_result["returncode"]
            )
        )
        return False, []

    if (
        "output" not in environment_dir_result
        or not environment_dir_result["output"]
    ):
        print(
            "Failed to get the environment directories, output: {}".format(
                environment_dir_result["output"]
            )
        )
        return False, []

    json_output = json.loads(environment_dir_result["output"])
    if "envs_dirs" not in json_output:
        return True, {}

    environment_directories = json_output["get"]["envs_dirs"]
    return True, environment_directories



import subprocess


def __format_output__(result, format_output_str=False):
    command_results = {}
    if hasattr(result, "args"):
        command_results.update(
            {"command": " ".join((getattr(result, "args")))}
        )
    if hasattr(result, "returncode"):
        command_results.update({"returncode": getattr(result, "returncode")})
    if hasattr(result, "stderr"):
        command_results.update({"error": getattr(result, "stderr")})
    if hasattr(result, "stdout"):
        command_results.update({"output": getattr(result, "stdout")})
    if hasattr(result, "wait"):
        command_results.update({"wait": getattr(result, "wait")})
    if hasattr(result, "communicate"):
        command_results.update({"communicate": getattr(result, "communicate")})
    if hasattr(result, "kill"):
        command_results.update({"kill": getattr(result, "kill")})

    if format_output_str:
        for key, value in command_results.items():
            if key == "returncode" or key == "error" or key == "output":
                if isinstance(value, bytes):
                    command_results[key] = value.decode("utf-8")
                else:
                    command_results[key] = str(value)
    return command_results


def run_popen(cmd, format_output_str=False, **run_kwargs):
    result = subprocess.Popen(cmd, **run_kwargs)
    return __format_output__(result, format_output_str=format_output_str)


def check_call(cmd, format_output_str=False, **kwargs):
    result = subprocess.check_call(cmd, **kwargs)
    return __format_output__(result, format_output_str=format_output_str)


def run(cmd, format_output_str=False, **run_kwargs):
    result = subprocess.run(cmd, **run_kwargs)
    return __format_output__(result, format_output_str=format_output_str)
